/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: AbstractVirtualMachine.java 325 2011-09-15 11:57:02Z chou $
 *
 */

package org.ow2.sirocco.vmm.agent.domain;

import java.util.Date;

import javax.management.ObjectName;

import org.ow2.sirocco.vmm.api.ResourcePartitionMXBean;
import org.ow2.sirocco.vmm.api.VMMException;
import org.ow2.sirocco.vmm.api.VirtualMachineMXBean;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetric;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetricInfo;

public abstract class AbstractVirtualMachine extends ManagedResource implements VirtualMachineMXBean {
    private ResourcePartition resourcePartition;

    public AbstractVirtualMachine(final ObjectName objectName) {
        super(objectName);
    }

    @Override
    public ResourcePartitionMXBean getResourcePartition() {
        return this.resourcePartition;
    }

    public void setResourcePartition(final ResourcePartitionMXBean partition) {
        if (this.resourcePartition != null) {
            this.resourcePartition.removeVirtualMachine(this);
        }
        this.resourcePartition = (ResourcePartition) partition;
        this.resourcePartition.addVirtualMachine(this);
    }

    public void removeFromResourcePartition() {
        this.resourcePartition.removeVirtualMachine(this);
    }

    public PerfMetricInfo[] listPerfMetricInfos() throws VMMException {
        ServerPool sp = (ServerPool) this.getHostMBean().getServerPool();
        if (sp.getMonitoringDriver() == null) {
            ManagedResource.logger.error("Monitoring service missing");
            return null;
        }
        return sp.getMonitoringDriver().listPerfMetrics(this);

    }

    public PerfMetric getPerfMetric(final String metricName) throws VMMException {
        ServerPool sp = (ServerPool) this.getHostMBean().getServerPool();
        if (sp.getMonitoringDriver() == null) {
            ManagedResource.logger.error("Monitoring service missing");
            return null;
        }
        return sp.getMonitoringDriver().getPerfMetric(this, metricName);

    }

    public PerfMetric[] getPerfMetrics(final String metricName, final Date startTime, final Date endTime, final long period,
        final ConsolidationFunction conFun) throws VMMException {

        ServerPool sp = (ServerPool) this.getHostMBean().getServerPool();
        if (sp.getMonitoringDriver() == null) {
            ManagedResource.logger.error("Monitoring service missing");
            return null;
        }

        return sp.getMonitoringDriver().getPerfMetrics(this, metricName, startTime, endTime, period, conFun.toString());
    }

}
