/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: FirstFitVMPlacementPolicy.java 79 2011-06-30 08:04:22Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.agent.domain.policy;

import java.util.List;
import java.util.Random;

import org.apache.log4j.Logger;
import org.ow2.sirocco.vmm.api.HostMXBean;
import org.ow2.sirocco.vmm.api.ServerPoolMXBean;
import org.ow2.sirocco.vmm.api.VMConfigSpec;
import org.ow2.sirocco.vmm.api.VMMException;
import org.ow2.sirocco.vmm.api.VirtualMachineConfigSpec;

/**
 * FirstFitVMPlacementPolicy is a VM placement policy which chooses the first
 * host which can fit the virtual machine configuration.
 */
public class RandomPlacementPolicy implements VMPlacementPolicy {
    static Logger logger = Logger.getLogger(RandomPlacementPolicy.class);

    private Random random = new Random();

    public RandomPlacementPolicy() {
    }

    @Override
    public HostMXBean placeVM(final ServerPoolMXBean pool, final VirtualMachineConfigSpec vmSpec) {
        return this.placeVM(pool, vmSpec.getNumVCPUs(), vmSpec.getMemoryMB(), vmSpec.computeDiskSpaceRequirementMB());
    }

    @Override
    public HostMXBean placeVM(final ServerPoolMXBean pool, final VMConfigSpec vmSpec) {
        return this.placeVM(pool, vmSpec.getNumVCPU(), vmSpec.getMemorySizeMB(), vmSpec.getDiskSizeMB());
    }

    private HostMXBean placeVM(final ServerPoolMXBean serverPool, final int numCPUs, final long memoryMB, final int diskMB) {
        List<HostMXBean> hosts = serverPool.getManagedHosts();
        HostMXBean host = hosts.get(this.random.nextInt(hosts.size()));
        try {
            RandomPlacementPolicy.logger.debug("Random VM placement policy selected host: " + host.getHostName());
        } catch (VMMException e) {
        }
        return host;
    }

}
