/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: RoomForGrowthVMPlacementPolicy.java 198 2011-07-26 11:18:30Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.agent.domain.policy;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;
import org.ow2.sirocco.vmm.agent.domain.AbstractHost;
import org.ow2.sirocco.vmm.api.HostMXBean;
import org.ow2.sirocco.vmm.api.HostMXBean.HostPowerState;
import org.ow2.sirocco.vmm.api.ServerPoolMXBean;
import org.ow2.sirocco.vmm.api.VMConfigSpec;
import org.ow2.sirocco.vmm.api.VMMException;
import org.ow2.sirocco.vmm.api.VirtualMachineConfigSpec;

/**
 * RoomForGrowthVMPlacementPolicy is a VM placement policy which attempts to
 * spread virtual machines on the hosts of server pool so as to leave maximmum
 * room for growth
 */
public class RoomForGrowthVMPlacementPolicy implements VMPlacementPolicy {
    static Logger logger = Logger.getLogger(RoomForGrowthVMPlacementPolicy.class);

    public RoomForGrowthVMPlacementPolicy() {
    }

    @Override
    public HostMXBean placeVM(final ServerPoolMXBean pool, final VirtualMachineConfigSpec vmSpec) {
        return this.placeVM(pool, vmSpec.getNumVCPUs(), vmSpec.getMemoryMB(), vmSpec.computeDiskSpaceRequirementMB());
    }

    @Override
    public HostMXBean placeVM(final ServerPoolMXBean pool, final VMConfigSpec vmSpec) {
        return this.placeVM(pool, vmSpec.getNumVCPU(), vmSpec.getMemorySizeMB(), vmSpec.getDiskSizeMB());
    }

    private HostMXBean placeVM(final ServerPoolMXBean serverPool, final int numCPUs, final long memoryMB, final int diskMB) {
        List<HostMXBean> hosts = serverPool.getManagedHosts();

        List<AbstractHost> candidateHostList = new ArrayList<AbstractHost>();
        for (HostMXBean h : hosts) {
            AbstractHost host = (AbstractHost) h;
            try {
                if (host.getHostState() == HostPowerState.RUNNING) {
                    candidateHostList.add(host);
                }
            } catch (VMMException ex) {
                RoomForGrowthVMPlacementPolicy.logger.error(ex);
            }
        }
        Collections.sort(candidateHostList, new Comparator<AbstractHost>() {
            public int compare(final AbstractHost h1, final AbstractHost h2) {
                long usedMem1 = h1.getReservedMemoryMB();
                long usedMem2 = h2.getReservedMemoryMB();
                return (int) (usedMem1 - usedMem2);
            }
        });

        try {
            for (AbstractHost host : candidateHostList) {
                RoomForGrowthVMPlacementPolicy.logger.debug("\t" + host.getHostName() + " usedMemMB="
                    + host.getReservedMemoryMB() + "MB");
            }

            RoomForGrowthVMPlacementPolicy.logger.debug("Room-for-growth policy selected host: "
                + (candidateHostList.size() > 0 ? candidateHostList.get(0).getHostName() : "none"));
        } catch (VMMException ex) {
        }
        return candidateHostList.size() > 0 ? candidateHostList.get(0) : null;
    }

}
