/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VMPlacementPolicy.java 79 2011-06-30 08:04:22Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.agent.domain.policy;

import org.ow2.sirocco.vmm.api.HostMXBean;
import org.ow2.sirocco.vmm.api.ServerPoolMXBean;
import org.ow2.sirocco.vmm.api.VMConfigSpec;
import org.ow2.sirocco.vmm.api.VirtualMachineConfigSpec;

/**
 * The VMPlacementPolicy interface abstracts a VM placement policy which decides
 * on which host of server pool a new virtual machine should be place. Every
 * server pool is associated with one VM placement policy.
 */
public interface VMPlacementPolicy {

    /**
     * Returns the host of a server pool where the virtual machine whose
     * configuration is provided as input should be placed.
     * 
     * @param pool the server pool where the virtual machine is to placed
     * @param vmSpec the configuration of the virtual machine
     * @return the host where the virtual machine should be placed or null if
     *         the policy is unable to place the virtual machine due to lack of
     *         resources
     */
    HostMXBean placeVM(ServerPoolMXBean pool, VMConfigSpec vmSpec);

    HostMXBean placeVM(ServerPoolMXBean pool, VirtualMachineConfigSpec vmSpec);

}
