/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: LocalExec.java 13 2011-06-14 21:21:51Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.agent.driver.util;

import java.io.IOException;
import java.io.InputStream;

import org.apache.log4j.Logger;
import org.ow2.sirocco.vmm.agent.driver.util.RemoteExec.Result;

public class LocalExec {

    static Logger logger = Logger.getLogger(RemoteExec.class);

    /***************************************************************************
     * @param cmd :command to execute
     * @return :return from running command cmd
     */
    static public Result command(final String command) throws Exception {
        StringBuffer outBuf = null;
        StringBuffer errBuf = null;
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        InputStream inStream = null;
        InputStream errStream = null;

        Result result = new Result();

        int ch;
        try {
            process = runtime.exec(command);
            inStream = process.getInputStream();
            errStream = process.getErrorStream();

            outBuf = new StringBuffer();
            errBuf = new StringBuffer();

            while ((ch = inStream.read()) != -1) {
                outBuf.append((char) ch + "");
            }

            while ((ch = errStream.read()) != -1) {
                errBuf.append((char) ch + "");
            }

            result.output = outBuf.toString();
            result.error = errBuf.toString();

        } catch (IOException e) {
            result.exitCode = -1;
            result.error = e.getMessage();
            result.output = "";
        } finally {
            try {
                if (process != null) {
                    result.exitCode = process.exitValue();
                    process.destroy();
                }
                if (inStream != null) {
                    inStream.close();
                }
            } catch (Exception e) {
            }
        }

        return result;
    }
}
