/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: AgentCommon.java 13 2011-06-14 21:21:51Z dangtran $
 *
 */

package org.ow2.sirocco.vmm.agent.main;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.apache.log4j.Logger;

/**
 * AgentCommon provides access to the VMM agent MBeanServer singleton and to
 * various agent-wide services.
 */
public class AgentCommon {
    static Logger logger = Logger.getLogger(AgentCommon.class);

    protected static MBeanServer mbs;

    protected AgentCommon() {
        // AgentCommon.mbs = MBeanServerFactory.createMBeanServer();
    }

    /**
     * Returns the agent's MBeanServer
     * 
     * @return the agent's MBeanServer
     */
    public static MBeanServer getMBeanServer() {
        if (AgentCommon.mbs == null) {
            AgentCommon.mbs = MBeanServerFactory.createMBeanServer();
        }
        return AgentCommon.mbs;
    }

    private static Map<String, String> scriptMap = new HashMap<String, String>();

    public static String getScript(final String folder, final String scriptName) {
        String script = AgentCommon.scriptMap.get(scriptName);
        if (script != null) {
            return script;
        }
        String filePath = AgentControl.vmmHome + File.separator + "etc" + File.separator + folder + File.separator + "scripts"
            + File.separator + scriptName;
        FileInputStream input = null;
        try {
            input = new FileInputStream(filePath);
            byte[] fileData = new byte[input.available()];
            input.read(fileData);
            script = new String(fileData, "UTF-8");
        } catch (IOException ex) {
            AgentCommon.logger.error("Failed to load script: " + scriptName);
            return "";
        } finally {
            try {
                input.close();
            } catch (Exception ex) {
            }
        }
        AgentCommon.scriptMap.put(scriptName, script);
        return script;
    }
}
