/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: MonitoringDriverAPI.java 325 2011-09-15 11:57:02Z chou $
 *
 */

package org.ow2.sirocco.vmm.agent.monitoring.api;

import java.util.Date;

import org.ow2.sirocco.vmm.agent.domain.ManagedResource;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetric;
import org.ow2.sirocco.vmm.api.monitoring.PerfMetricInfo;

public interface MonitoringDriverAPI {

    /**
     * Start and initialize the monitoring service
     * 
     * @param args
     */
    public void start(final String args[]);

    /**
     * Stop the monitoring service
     * 
     * @param args
     */
    public void stop();

    /**
     * Retrieve the list of metrics of this host
     * 
     * @param host : the name of the host
     * @return String[] : list of metrics of this host if there is or null
     */
    public PerfMetricInfo[] listPerfMetrics(ManagedResource resource);

    /**
     * Retrieve the value of the metadata's metric of this host
     * 
     * @param host : the name of the host
     * @param metadata : the name of the metric
     * @return PerfMetric : value of this metric if there is or null
     */
    public PerfMetric getPerfMetric(ManagedResource resource, String metadata);

    /**
     * Retrieve the value of the metadata's metric of this host between the date
     * startTime and endTime with a period
     * 
     * @param host : the name of the host
     * @param metadata : the name of the metric
     * @param startTime : the date where we start retrieve values
     * @param endTime : the date where we stop retrieve values
     * @param period : the period we retrieve values
     * @return String : list of values's metric of this host if there is or null
     */
    public PerfMetric[] getPerfMetrics(ManagedResource resource, String info, Date startTime, Date endTime, long period,
        String type);

    /**
     * Initialize variables when a hostname starts if needed
     * 
     * @param hostName
     */
    public void startHost(final String hostName);

    /**
     * Initialize variables when a hostname stop if needed
     * 
     * @param hostName
     */
    public void stopHost(final String hostName);

    /**
     * Initialize variables when a VM starts if needed
     * 
     * @param VMName
     */
    public void startVM(final String VMName);

    /**
     * Initialize variables when a VM stop if needed
     * 
     * @param VMName
     */
    public void stopVM(final String VMName);
}
